@@ -37,8 +37,8 @@ CONFIG_NAME = 'firewall-config'
 APPLET_NAME = 'firewall-applet'
 DATADIR = '/usr/share/' + DAEMON_NAME
 CONFIG_GLADE_NAME = CONFIG_NAME + '.glade'
-COPYRIGHT = '(C) 2010-2012 Red Hat, Inc.'
-VERSION = '0.3.14.2'
+COPYRIGHT = '(C) 2010-2015 Red Hat, Inc.'
+VERSION = '0.4.0'
 AUTHORS = [
     "Thomas Woerner <twoerner@redhat.com>",
     "Jiri Popelka <jpopelka@redhat.com>",
@@ -63,23 +63,42 @@ FIREWALLD_CONF = ETC_FIREWALLD + '/firewalld.conf'
 ETC_FIREWALLD_ZONES = ETC_FIREWALLD + '/zones'
 ETC_FIREWALLD_SERVICES = ETC_FIREWALLD + '/services'
 ETC_FIREWALLD_ICMPTYPES = ETC_FIREWALLD + '/icmptypes'
+ETC_FIREWALLD_IPSETS = ETC_FIREWALLD + '/ipsets'
 
 USR_LIB_FIREWALLD = '/usr/lib/firewalld'
 FIREWALLD_ZONES = USR_LIB_FIREWALLD + '/zones'
 FIREWALLD_SERVICES = USR_LIB_FIREWALLD + '/services'
 FIREWALLD_ICMPTYPES = USR_LIB_FIREWALLD + '/icmptypes'
+FIREWALLD_IPSETS = USR_LIB_FIREWALLD + '/ipsets'
 
 FIREWALLD_LOGFILE = '/var/log/firewalld'
 
+FIREWALLD_TEMPDIR = '/run/firewalld'
+
 FIREWALLD_DIRECT = ETC_FIREWALLD + '/direct.xml'
 
 LOCKDOWN_WHITELIST = ETC_FIREWALLD + '/lockdown-whitelist.xml'
 
 SYSCTL_CONFIG = '/etc/sysctl.conf'
 
+# commands used by backends
+COMMANDS = {
+    "ipv4":         "/usr/bin/iptables",
+    "ipv4-restore": "/usr/bin/iptables-restore",
+    "ipv6":         "/usr/bin/ip6tables",
+    "ipv6-restore": "/usr/bin/ip6tables-restore",
+    "eb":           "/usr/bin/ebtables",
+    "eb-restore":   "/usr/bin/ebtables-restore",
+    "ipset":        "/usr/bin/ipset",
+}
+
+LOG_DENIED_VALUES = [ "all", "unicast", "broadcast", "multicast", "off" ]
+
 # fallbacks: will be overloaded by firewalld.conf
 FALLBACK_ZONE = "public"
 FALLBACK_MINIMAL_MARK = 100
 FALLBACK_CLEANUP_ON_EXIT = True
 FALLBACK_LOCKDOWN = False
 FALLBACK_IPV6_RPFILTER = True
+FALLBACK_INDIVIDUAL_CALLS = False
+FALLBACK_LOG_DENIED = "off"
