@@ -25,6 +25,7 @@ from firewall.config import *
 from firewall import functions
 from firewall.core import ipXtables
 from firewall.core import ebtables
+from firewall.core import ipset
 from firewall.core import modules
 from firewall.core.fw_icmptype import FirewallIcmpType
 from firewall.core.fw_service import FirewallService
@@ -32,12 +33,14 @@ from firewall.core.fw_zone import FirewallZone
 from firewall.core.fw_direct import FirewallDirect
 from firewall.core.fw_config import FirewallConfig
 from firewall.core.fw_policies import FirewallPolicies
+from firewall.core.fw_ipset import FirewallIPSet
 from firewall.core.logger import log
 from firewall.core.io.firewalld_conf import firewalld_conf
 from firewall.core.io.direct import Direct
 from firewall.core.io.service import service_reader
 from firewall.core.io.icmptype import icmptype_reader
 from firewall.core.io.zone import zone_reader, Zone
+from firewall.core.io.ipset import ipset_reader
 from firewall.errors import *
 
 ############################################################################
@@ -56,6 +59,8 @@ class Firewall(object):
         self.ip6tables_enabled = True
         self._ebtables = ebtables.ebtables()
         self.ebtables_enabled = True
+        self._ipset = ipset.ipset()
+        self.ipset_enabled = True
 
         self._modules = modules.modules()
 
@@ -65,15 +70,17 @@ class Firewall(object):
         self.direct = FirewallDirect(self)
         self.config = FirewallConfig(self)
         self.policies = FirewallPolicies()
+        self.ipset = FirewallIPSet(self)
 
         self.__init_vars()
 
     def __repr__(self):
-        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)' % \
+        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r %r)' % \
             (self.__class__, self.ip4tables_enabled, self.ip6tables_enabled,
              self.ebtables_enabled, self._state, self._panic,
              self._default_zone, self._module_refcount, self._marks,
-             self._min_mark, self.cleanup_on_exit, self.ipv6_rpfilter_enabled)
+             self._min_mark, self.cleanup_on_exit, self.ipv6_rpfilter_enabled,
+             self.ipset_enabled, self._individual_calls, self._log_denied)
 
     def __init_vars(self):
         self._state = "INIT"
@@ -81,21 +88,27 @@ class Firewall(object):
         self._default_zone = ""
         self._module_refcount = { }
         self._marks = [ ]
-        self._min_mark = FALLBACK_MINIMAL_MARK # will be overloaded by firewalld.conf
+        # fallback settings will be overloaded by firewalld.conf
+        self._min_mark = FALLBACK_MINIMAL_MARK
         self.cleanup_on_exit = FALLBACK_CLEANUP_ON_EXIT
         self.ipv6_rpfilter_enabled = FALLBACK_IPV6_RPFILTER
+        self._individual_calls = FALLBACK_INDIVIDUAL_CALLS
+        self._log_denied = FALLBACK_LOG_DENIED
 
     def _check_tables(self):
         # check if iptables, ip6tables and ebtables are usable, else disable
-        if "filter" not in ipXtables.ip4tables_available_tables:
+        if self.ip4tables_enabled and \
+           "filter" not in ipXtables.ip4tables_available_tables:
             log.warning("iptables not usable, disabling IPv4 firewall.")
             self.ip4tables_enabled = False
 
-        if "filter" not in ipXtables.ip6tables_available_tables:
+        if self.ip6tables_enabled and \
+           "filter" not in ipXtables.ip6tables_available_tables:
             log.warning("ip6tables not usable, disabling IPv6 firewall.")
             self.ip6tables_enabled = False
 
-        if "filter" not in ebtables.ebtables_available_tables:
+        if self.ebtables_enabled and \
+           "filter" not in ebtables.ebtables_available_tables:
             log.error("ebtables not usable, disabling ethernet bridge firewall.")
             self.ebtables_enabled = False
 
@@ -103,6 +116,13 @@ class Firewall(object):
             log.fatal("No IPv4 and IPv6 firewall.")
             sys.exit(1)
 
+    def _start_check(self):
+        try:
+            x = self._ipset.list()
+        except:
+            log.error("ipset not usable, disabling ipset usage in firewall.")
+            self.ipset_enabled = False
+
     def _start(self):
         # initialize firewall
         default_zone = FALLBACK_ZONE
@@ -147,6 +167,24 @@ class Firewall(object):
             else:
                 log.debug1("IPV6 rpfilter is disabled")
 
+            if self._firewalld_conf.get("IndividualCalls"):
+                value = self._firewalld_conf.get("IndividualCalls")
+                if value is not None and value.lower() in [ "yes", "true" ]:
+                    log.debug1("IndividualCalls is enabled")
+                    self._individual_calls = True
+
+            if self._firewalld_conf.get("LogDenied"):
+                value = self._firewalld_conf.get("LogDenied")
+                if value is None or value.lower() == "no":
+                    self._log_denied = "off"
+                else:
+                    self._log_denied = value.lower()
+                    log.debug1("LogDenied is set to '%s'", self._log_denied)
+
+            if not self._individual_calls and \
+               not self._ebtables.restore_noflush_option:
+                log.debug1("ebtables-restore is not supporting the --noflush option, will therefore not be used")
+
         self.config.set_firewalld_conf(copy.deepcopy(self._firewalld_conf))
 
         # apply default rules
@@ -167,6 +205,10 @@ class Firewall(object):
         # copy policies to config interface
         self.config.set_policies(copy.deepcopy(self.policies))
 
+        # load ipset files
+        self._loader(FIREWALLD_IPSETS, "ipset")
+        self._loader(ETC_FIREWALLD_IPSETS, "ipset")
+
         # load icmptype files
         self._loader(FIREWALLD_ICMPTYPES, "icmptype")
         self._loader(ETC_FIREWALLD_ICMPTYPES, "icmptype")
@@ -198,6 +240,9 @@ class Firewall(object):
         if error:
             sys.exit(1)
 
+        # apply settings for loaded ipsets
+        self.ipset.apply_ipsets(self._individual_calls)
+
         # apply settings for loaded zones
         self.zone.apply_zones()
 
@@ -210,8 +255,7 @@ class Firewall(object):
             except Exception as msg:
                 log.debug1("Failed to load direct rules file '%s': %s",
                            FIREWALLD_DIRECT, msg)
-            else:
-                self.direct.set_permanent_config(obj)
+        self.direct.set_permanent_config(obj)
         self.config.set_direct(copy.deepcopy(obj))
 
         # check if default_zone is a valid zone
@@ -235,6 +279,7 @@ class Firewall(object):
         self._state = "RUNNING"
 
     def start(self):
+        self._start_check()
         self._check_tables()
         self._flush()
         self._set_policy("ACCEPT")
@@ -275,6 +320,8 @@ class Firewall(object):
                                    orig_obj.name, orig_obj.path,
                                    orig_obj.filename)
                         self.icmptype.remove_icmptype(orig_obj.name)
+                    elif obj.path.startswith(ETC_FIREWALLD):
+                        obj.default = True
                     self.icmptype.add_icmptype(obj)
                     # add a deep copy to the configuration interface
                     self.config.add_icmptype(copy.deepcopy(obj))
@@ -286,6 +333,8 @@ class Firewall(object):
                                    orig_obj.name, orig_obj.path,
                                    orig_obj.filename)
                         self.service.remove_service(orig_obj.name)
+                    elif obj.path.startswith(ETC_FIREWALLD):
+                        obj.default = True
                     self.service.add_service(obj)
                     # add a deep copy to the configuration interface
                     self.config.add_service(copy.deepcopy(obj))
@@ -312,6 +361,9 @@ class Firewall(object):
                                        reader_type,
                                        orig_obj.name, orig_obj.path,
                                        orig_obj.filename)
+                    elif obj.path.startswith(ETC_FIREWALLD):
+                        obj.default = True
+                        config_obj.default = True
                     self.config.add_zone(config_obj)
                     if combine:
                         log.debug1("  Combining %s '%s' ('%s/%s')",
@@ -320,6 +372,19 @@ class Firewall(object):
                         combined_zone.combine(obj)
                     else:
                         self.zone.add_zone(obj)
+                elif reader_type == "ipset":
+                    obj = ipset_reader(filename, path)
+                    if obj.name in self.ipset.get_ipsets():
+                        orig_obj = self.ipset.get_ipset(obj.name)
+                        log.debug1("  Overloads %s '%s' ('%s/%s')", reader_type,
+                                   orig_obj.name, orig_obj.path,
+                                   orig_obj.filename)
+                        self.ipset.remove_ipset(orig_obj.name)
+                    elif obj.path.startswith(ETC_FIREWALLD):
+                        obj.default = True
+                    self.ipset.add_ipset(obj)
+                    # add a deep copy to the configuration interface
+                    self.config.add_ipset(copy.deepcopy(obj))
                 else:
                     log.fatal("Unknown reader type %s", reader_type)
             except FirewallError as msg:
@@ -346,6 +411,7 @@ class Firewall(object):
         self.icmptype.cleanup()
         self.service.cleanup()
         self.zone.cleanup()
+        self.ipset.cleanup()
         self.config.cleanup()
         self.direct.cleanup()
         self.policies.cleanup()
@@ -381,10 +447,22 @@ class Firewall(object):
         else:
             append_delete = { True: "-A", False: "-D", }
 
+        _rules = { }
         # appends rules
         # returns None if all worked, else (cleanup rules, error message)
         for i,value in enumerate(rules):
-            if len(value) == 3:
+            table = chain = None
+            if len(value) == 5:
+                (ipv, table, chain, rule, insert) = value
+                # drop insert rule number if it exists
+                if insert and not enable and isinstance(rule[1], int):
+                    rule.pop(1)
+            elif len(value) == 4:
+                (ipv, table, chain, rule) = value
+                # drop insert rule number if it exists
+                if insert and not enable and isinstance(rule[1], int):
+                    rule.pop(1)
+            elif len(value) == 3:
                 (ipv, rule, insert) = value
             else:
                 (ipv, rule) = value
@@ -393,70 +471,66 @@ class Firewall(object):
             if insert and not enable and isinstance(rule[1], int):
                 rule.pop(1)
 
-            table = None
-            for t in ipXtables.BUILT_IN_CHAINS.keys():
-                if t in rule:
-                    table = t
             if table and not self.is_table_available(ipv, table):
                 if ((ipv == "ipv4" and self.ip4tables_enabled) or
                     (ipv == "ipv6" and self.ip6tables_enabled)):
                     log.error("Unable to add %s into %s %s" % (rule, ipv, table))
                 continue
 
-            # run
-            try:
-                self.rule(ipv, [ append_delete[enable], ] + rule)
-            except Exception as msg:
-                log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
-                log.error(msg)
-                return (rules[:i], msg) # cleanup rules and error message
-        return None
-
-    def handle_rules2(self, rules, enable, insert=False):
-        if insert:
-            append_delete = { True: "-I", False: "-D", }
-        else:
-            append_delete = { True: "-A", False: "-D", }
-
-        # appends rules
-        # returns None if all worked, else (cleanup rules, error message)
-        for i,value in enumerate(rules):
-            if len(value) == 5:
-                (ipv, table, chain, rule, insert) = value
+            if table != None:
+                _rule = [ "-t", table, append_delete[enable], ]
             else:
-                (ipv, table, chain, rule) = value
+                _rule = [ append_delete[enable], ]
+            if chain != None:
+                _rule.append(chain)
+            _rule += [ "%s" % item for item in rule ]
 
-            # drop insert rule number if it exists
-            if insert and not enable and isinstance(rule[1], int):
-                rule.pop(1)
+            if self._individual_calls or \
+               (ipv == "eb" and not self._ebtables.restore_noflush_option):
+                ## run
+                try:
+                    self.rule(ipv, _rule)
+                except Exception as msg:
+                    log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
+                    log.error(msg)
+                    return (rules[:i], msg) # cleanup rules and error message
+            else:
+                _rules.setdefault(ipv, []).append(_rule)
 
-            if not self.is_table_available(ipv, table):
-                if ((ipv == "ipv4" and self.ip4tables_enabled) or
-                    (ipv == "ipv6" and self.ip6tables_enabled)):
-                    log.error("Unable to add %s into %s %s" % (rule, ipv, table))
-                continue
+        try:
+            for ipv in _rules:
+                self.rules(ipv, _rules[ipv])
+        except Exception as msg:
+            log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
+            log.error(msg)
+            return ([ ], msg) # no cleanup rules and error message
 
-            # run
-            try:
-                self.rule(ipv, [ "-t", table,
-                                 append_delete[enable], chain, ] + rule)
-            except Exception as msg:
-                log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
-                log.error(msg)
-                return (rules[:i], msg) # cleanup rules and error message
         return None
 
     def handle_chains(self, rules, enable):
         new_delete = { True: "-N", False: "-X" }
 
+        _rules = { }
         # appends chains
         # returns None if all worked, else (cleanup chains, error message)
         for i,(ipv, rule) in enumerate(rules):
-            try:
-                self.rule(ipv, [ new_delete[enable], ] + rule)
-            except Exception as msg:
-                log.error(msg)
-                return (rules[:i], msg) # cleanup chains and error message
+            _rule = [ new_delete[enable], ] + rule
+            if self._individual_calls or \
+               (ipv == "eb" and not self._ebtables.restore_noflush_option):
+                try:
+                    self.rule(ipv, _rule)
+                except Exception as msg:
+                    log.error(msg)
+                    return (rules[:i], msg) # cleanup chains and error message
+            else:
+                _rules.setdefault(ipv, []).append(_rule)
+        try:
+            for ipv in _rules:
+                self.rules(ipv, _rules[ipv])
+        except Exception as msg:
+            log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
+            log.error(msg)
+            return ([ ], msg) # no cleanup rules and error message
         return None
 
     def handle_modules(self, modules, enable):
@@ -490,25 +564,37 @@ class Firewall(object):
 
     # apply default rules
     def __apply_default_rules(self, ipv):
-        if ipv in [ "ipv4", "ipv6" ]:
-            default_rules = ipXtables.DEFAULT_RULES
-        else:
-            default_rules = ebtables.DEFAULT_RULES
+        default_rules = { }
 
+        if ipv in [ "ipv4", "ipv6" ]:
+            x = ipXtables
+        else:
+            x = ebtables
+        for table in x.DEFAULT_RULES:
+            default_rules[table] = x.DEFAULT_RULES[table][:]
+
+        if self._log_denied != "off":
+            for table in x.LOG_RULES:
+                default_rules.setdefault(table, []).extend(x.LOG_RULES[table])
+
+        rules = { }
         for table in default_rules:
             if not self.is_table_available(ipv, table):
                 continue
             prefix = [ "-t", table ]
             for rule in default_rules[table]:
-                _rule = prefix + rule.split()
-                self.rule(ipv, _rule)
+                if type(rule) == list:
+                    _rule = prefix + rule
+                else:
+                    _rule = prefix + functions.splitArgs(rule)
+                if self._individual_calls or \
+                   (ipv == "eb" and not self._ebtables.restore_noflush_option):
+                    self.rule(ipv, _rule)
+                else:
+                    rules.setdefault(ipv, []).append(_rule)
 
-#                try:
-#                except Exception as msg:
-# TODO: better handling of init error
-#                    if "Chain already exists." in msg:
-#                        continue
-#                    raise FirewallError, <code>
+        for ipv in rules:
+            self.rules(ipv, rules[ipv])
 
     def _apply_default_rules(self):
         for ipv in [ "ipv4", "ipv6", "eb" ]:
@@ -516,35 +602,49 @@ class Firewall(object):
 
         if self.ipv6_rpfilter_enabled and \
            self.is_table_available("ipv6", "raw"):
-            rule = [ "-t", "raw", "-I", "PREROUTING", "1",
-                     "-p", "icmpv6", "--icmpv6-type=router-advertisement",
-                     "-j", "ACCEPT" ]       # RHBZ#1058505
-            self.rule("ipv6", rule)
-            rule = [ "-t", "raw", "-I", "PREROUTING", "2",
-                     "-m", "rpfilter", "--invert", "-j", "DROP" ]
-            try:
-                self.rule("ipv6", rule)
-            except ValueError:    # some problem with ip6t_rpfilter module ?
-                rule = [ "-t", "raw", "-D", "PREROUTING", "1"]
-                self.rule("ipv6", rule)
+            # here is no check for ebtables.restore_noflush_option needed
+            # as ebtables is not used in here
+            rules = [
+                ("ipv6", [ "PREROUTING", 1, "-t", "raw",
+                           "-p", "icmpv6", "--icmpv6-type=router-advertisement",
+                           "-j", "ACCEPT" ]), # RHBZ#1058505
+                ("ipv6", [ "PREROUTING", 2, "-t", "raw",
+                           "-m", "rpfilter", "--invert", "-j", "DROP" ]),
+            ]
+            if self._log_denied != "off":
+                rules.append(("ipv6", [ "PREROUTING", 2, "-t", "raw",
+                                        "-m", "rpfilter", "--invert",
+                                        "-j", "LOG",
+                                        "--log-prefix", "rpfilter_DROP: " ]))
+            # handle rules
+            ret = self.handle_rules(rules, True, insert=True)
+            if ret:
+                (cleanup_rules, msg) = ret
+                self.handle_rules(cleanup_rules, False)
+                log.error(msg)
 
     # flush and policy
 
     def _flush(self):
         if self.ip4tables_enabled:
-            self._ip4tables.flush()
+            self._ip4tables.flush(individual=self._individual_calls)
         if self.ip6tables_enabled:
-            self._ip6tables.flush()
+            self._ip6tables.flush(individual=self._individual_calls)
         if self.ebtables_enabled:
-            self._ebtables.flush()
+            self._ebtables.flush(individual=(self._individual_calls or \
+                                             not self._ebtables.restore_noflush_option))
 
     def _set_policy(self, policy, which="used"):
         if self.ip4tables_enabled:
-            self._ip4tables.set_policy(policy, which)
+            self._ip4tables.set_policy(policy, which,
+                                       individual=self._individual_calls)
         if self.ip6tables_enabled:
-            self._ip6tables.set_policy(policy, which)
+            self._ip6tables.set_policy(policy, which,
+                                       individual=self._individual_calls)
         if self.ebtables_enabled:
-            self._ebtables.set_policy(policy, which)
+            self._ebtables.set_policy(policy, which,
+                                      individual=(self._individual_calls or \
+                                                  not self._ebtables.restore_noflush_option))
 
     # rule function used in handle_ functions
 
@@ -574,6 +674,31 @@ class Firewall(object):
                 raise FirewallError(INVALID_IPV,
                                     "'%s' not in {'ipv4'|'ipv6'}" % ipv)
 
+        # replace %%LOGTYPE%%
+        try:
+            i = rule.index("%%LOGTYPE%%")
+        except:
+            pass
+        else:
+            if self._log_denied == "off":
+                return ""
+            if ipv not in [ "ipv4", "ipv6" ]:
+                raise FirewallError(INVALID_IPV,
+                                    "'%s' not in {'ipv4'|'ipv6'}" % ipv)
+            if self._log_denied in [ "unicast", "broadcast", "multicast" ]:
+                rule[i:i+1] = [ "-m", "pkttype", "--pkt-type",
+                                self._log_denied ]
+            else:
+                rule.pop(i)
+
+        # remove leading and trailing '"' for use with execve
+        i = 0
+        while i < len(rule):
+            x = rule[i]
+            if len(x) > 2 and x[0] == '"' and x[-1] == '"':
+                rule[i] = x[1:-1]
+            i += 1
+
         if ipv == "ipv4":
             # do not call if disabled
             if self.ip4tables_enabled:
@@ -592,6 +717,73 @@ class Firewall(object):
 
         return ""
 
+    def rules(self, ipv, rules):
+        _rules = [ ]
+
+        for rule in rules:
+            # replace %%REJECT%%
+            try:
+                i = rule.index("%%REJECT%%")
+            except:
+                pass
+            else:
+                if ipv in [ "ipv4", "ipv6" ]:
+                    rule[i:i+1] = [ "REJECT", "--reject-with",
+                                    ipXtables.DEFAULT_REJECT_TYPE[ipv] ]
+                else:
+                    raise FirewallError(EBTABLES_NO_REJECT,
+                                        "'%s' not in {'ipv4'|'ipv6'}" % ipv)
+
+            # replace %%ICMP%%
+            try:
+                i = rule.index("%%ICMP%%")
+            except:
+                pass
+            else:
+                if ipv in [ "ipv4", "ipv6" ]:
+                    rule[i] = ipXtables.ICMP[ipv]
+                else:
+                    raise FirewallError(INVALID_IPV,
+                                        "'%s' not in {'ipv4'|'ipv6'}" % ipv)
+
+            # replace %%LOGTYPE%%
+            try:
+                i = rule.index("%%LOGTYPE%%")
+            except:
+                pass
+            else:
+                if self._log_denied == "off":
+                    continue
+                if ipv not in [ "ipv4", "ipv6" ]:
+                    raise FirewallError(INVALID_IPV,
+                                    "'%s' not in {'ipv4'|'ipv6'}" % ipv)
+                if self._log_denied in [ "unicast", "broadcast",
+                                         "multicast" ]:
+                    rule[i:i+1] = [ "-m", "pkttype", "--pkt-type",
+                                    self._log_denied ]
+                else:
+                    rule.pop(i)
+
+            _rules.append(rule)
+
+        if ipv == "ipv4":
+            # do not call if disabled
+            if self.ip4tables_enabled:
+                return self._ip4tables.set_rules(_rules)
+        elif ipv == "ipv6":
+            # do not call if disabled
+            if self.ip6tables_enabled:
+                return self._ip6tables.set_rules(_rules)
+        elif ipv == "eb":
+            # do not call if disabled
+            if self.ebtables_enabled:
+                return self._ebtables.set_rules(_rules)
+        else:
+            raise FirewallError(INVALID_IPV,
+                                "'%s' not in {'ipv4'|'ipv6'|'eb'}" % ipv)
+
+        return ""
+
     # check functions
 
     def check_panic(self):
@@ -628,7 +820,7 @@ class Firewall(object):
                 log.debug1("'%s': range start >= end" % port)
             raise FirewallError(INVALID_PORT, port)
 
-    def check_protocol(self, protocol):
+    def check_tcpudp(self, protocol):
         if not protocol:
             raise FirewallError(MISSING_PROTOCOL)
         if not protocol in [ "tcp", "udp" ]:
@@ -687,6 +879,10 @@ class Firewall(object):
         # default zone if it changed
         _new_dz = self.get_default_zone()
         if _new_dz != _old_dz:
+            # if_new_dz has been introduced with the reload, we need to add it
+            # https://github.com/t-woerner/firewalld/issues/53
+            if _new_dz not in _zone_interfaces:
+                _zone_interfaces[_new_dz] = { }
             # default zone changed. Move interfaces from old default zone to 
             # the new one.
             for iface, settings in list(_zone_interfaces[_old_dz].items()):
@@ -756,6 +952,26 @@ class Firewall(object):
     def query_panic_mode(self):
         return self._panic
 
+    # LOG DENIED
+
+    def get_log_denied(self):
+        return self._log_denied
+
+    def set_log_denied(self, value):
+        if value not in LOG_DENIED_VALUES:
+            raise FirewallError(INVALID_VALUE, "'%s', choose from '%s'" % \
+                                (value, "','".join(LOG_DENIED_VALUES)))
+
+        if value != self.get_log_denied():
+            self._log_denied = value
+            self._firewalld_conf.set("LogDenied", value)
+            self._firewalld_conf.write()
+
+            # now reload the firewall
+            self.reload()
+        else:
+            raise FirewallError(ALREADY_SET, value)
+
     # DEFAULT ZONE
 
     def get_default_zone(self):
