@@ -25,15 +25,7 @@ from firewall import functions
 from firewall.fw_types import *
 from firewall.core import ipXtables
 from firewall.core import ebtables
-from firewall.core import modules
-from firewall.core.fw_icmptype import FirewallIcmpType
-from firewall.core.fw_service import FirewallService
-from firewall.core.fw_zone import FirewallZone
 from firewall.core.logger import log
-from firewall.core.io.firewalld_conf import firewalld_conf
-from firewall.core.io.service import service_reader
-from firewall.core.io.icmptype import icmptype_reader
-from firewall.core.io.zone import zone_reader
 from firewall.errors import *
 
 ############################################################################
@@ -160,7 +152,10 @@ class FirewallDirect(object):
         if chain in our_chains:
             raise FirewallError(BUILTIN_CHAIN,
                  "chain '%s' is reserved" % chain)
-
+        if ipv in [ "ipv4", "ipv6" ]:
+            if self._fw.zone.zone_from_chain(ipv, table, chain) != None:
+                raise FirewallError(INVALID_CHAIN,
+                                    "Chain '%s' is reserved" % chain)
 
     # DIRECT CHAIN
 
@@ -235,8 +230,10 @@ class FirewallDirect(object):
 
     def __rule(self, enable, ipv, table, chain, priority, args):
         self._check_ipv_table(ipv, table)
+        if ipv in [ "ipv4", "ipv6" ]:
+            self._fw.zone.create_zone_base_by_chain(ipv, table, chain)
+
         _chain = chain
-        # use "%s_chain" for built-in chains
 
         if ipv in [ "ipv4", "ipv6" ]:
             _CHAINS = ipXtables.BUILT_IN_CHAINS
@@ -395,6 +392,29 @@ class FirewallDirect(object):
 
         if enable:
             self.check_passthrough(args)
+            # try to find out if a zone chain should be used
+            if ipv in [ "ipv4", "ipv6" ]:
+                table = "filter"
+                try:
+                    i = args.index("-t")
+                except:
+                    pass
+                else:
+                    if len(args) >= i+1:
+                        table = args[i+1]
+                chain = None
+                for opt in [ "-A", "--append",
+                             "-I", "--insert",
+                             "-N", "--new-chain" ]:
+                    try:
+                        i = args.index(opt)
+                    except:
+                        pass
+                    else:
+                        if len(args) >= i+1:
+                            chain = args[i+1]
+                if table and chain:
+                    self._fw.zone.create_zone_base_by_chain(ipv, table, chain)
             _args = args
         else:
             _args = self.reverse_passthrough(args)
