@@ -99,7 +99,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         elif property_name == "default":
             return self.obj.default
         elif property_name == "builtin":
-            return self.config.is_builtin_zone(self.obj)
+            return self.obj.builtin
         else:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.AccessDenied: "
@@ -123,6 +123,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
             'filename': self.obj.filename,
             'path': self.obj.path,
             'default': self.obj.default,
+            'builtin': self.obj.builtin,
         }
 
     @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
@@ -158,7 +159,13 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         """get settings for zone
         """
         log.debug1("config.zone.%d.getSettings()", self.id)
-        return self.config.get_zone_config(self.obj)
+        settings = self.config.get_zone_config(self.obj)
+        if settings[4] == DEFAULT_ZONE_TARGET:
+            # convert to list, fix target, convert back to tuple
+            _settings = list(settings)
+            _settings[4] = "default"
+            settings = tuple(_settings)
+        return settings
 
     def _checkDuplicateInterfacesSources(self, settings):
         """Assignment of interfaces/sources to zones is different from other
@@ -186,6 +193,11 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         settings = dbus_to_python(settings)
         log.debug1("config.zone.%d.update('...')", self.id)
         self.parent.accessCheck(sender)
+        if settings[4] == "default":
+            # convert to list, fix target, convert back to tuple
+            _settings = list(settings)
+            _settings[4] = DEFAULT_ZONE_TARGET
+            settings = tuple(_settings)
         self._checkDuplicateInterfacesSources(settings)
         self.obj = self.config.set_zone_config(self.obj, settings)
         self.Updated(self.obj.name)
@@ -431,6 +443,57 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                    protocol)
         return (port,protocol) in self.getSettings()[6]
 
+    # protocol
+
+    @dbus_service_method(DBUS_INTERFACE_CONFIG_ZONE, out_signature='as')
+    @dbus_handle_exceptions
+    def getProtocols(self, sender=None):
+        log.debug1("config.zone.%d.getProtocols()", self.id)
+        return self.getSettings()[13]
+
+    @dbus_service_method(DBUS_INTERFACE_CONFIG_ZONE, in_signature='as')
+    @dbus_handle_exceptions
+    def setProtocols(self, protocols, sender=None):
+        protocols = dbus_to_python(protocols, list)
+        log.debug1("config.zone.%d.setProtocols('[%s]')", self.id,
+                   ",".join(protocols))
+        self.parent.accessCheck(sender)
+        settings = list(self.getSettings())
+        settings[13] = protocols
+        self.update(settings)
+
+    @dbus_service_method(DBUS_INTERFACE_CONFIG_ZONE, in_signature='s')
+    @dbus_handle_exceptions
+    def addProtocol(self, protocol, sender=None):
+        protocol = dbus_to_python(protocol, str)
+        log.debug1("config.zone.%d.addProtocol('%s')", self.id, protocol)
+        self.parent.accessCheck(sender)
+        settings = list(self.getSettings())
+        if protocol in settings[13]:
+            raise FirewallError(ALREADY_ENABLED, protocol)
+        settings[13].append(protocol)
+        self.update(settings)
+
+    @dbus_service_method(DBUS_INTERFACE_CONFIG_ZONE, in_signature='s')
+    @dbus_handle_exceptions
+    def removeProtocol(self, protocol, sender=None):
+        protocol = dbus_to_python(protocol, str)
+        log.debug1("config.zone.%d.removeProtocol('%s')", self.id, protocol)
+        self.parent.accessCheck(sender)
+        settings = list(self.getSettings())
+        if protocol not in settings[13]:
+            raise FirewallError(NOT_ENABLED, protocol)
+        settings[13].remove(protocol)
+        self.update(settings)
+
+    @dbus_service_method(DBUS_INTERFACE_CONFIG_ZONE, in_signature='s',
+                         out_signature='b')
+    @dbus_handle_exceptions
+    def queryProtocol(self, protocol, sender=None):
+        protocol = dbus_to_python(protocol, str)
+        log.debug1("config.zone.%d.queryProtocol('%s')", self.id, protocol)
+        return protocol in self.getSettings()[13]
+
     # icmp block
 
     @dbus_service_method(DBUS_INTERFACE_CONFIG_ZONE, out_signature='as')
